#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see <http://www.gnu.org/licenses/>.

    @author: X3n0m0rph59
"""

import sys, getopt, re, os, syslog, signal, time

from module.remote.thriftbackend.ThriftClient import ThriftClient, NoConnection,\
                                                     NoSSL, WrongLogin

from BaseHTTPServer import *
from cgi import FieldStorage

from urllib import unquote
from base64 import standard_b64decode
from binascii import unhexlify

try:
    from Crypto.Cipher import AES
except:
    pass

from imp import find_module
from os.path import exists
from urllib import quote

import subprocess


################################################################################

__app_name__ = "pyLoad Click'n'Load Proxy" 
__version__  = "0.1"

__log_verbosity__ = 3
__pid_file__ = "/var/run/pyLoadCNLProxy.pid"


################################################################################

class SimpleLogger():
    CONSOLE = 0
    SYSLOG = 1
    
    syslog_prios={'FATAL':syslog.LOG_ERR, 
                 'ERROR':syslog.LOG_ERR,
                 'WARNING':syslog.LOG_ERR,
                 'INFO':syslog.LOG_INFO,
                 'DEBUG':syslog.LOG_DEBUG}
    
    def __init__(self, target=CONSOLE, verbosity=__log_verbosity__):
        self.target = target
        self.verbosity = verbosity        
    
    def _log(self, severity, msg):
        if self.target == self.SYSLOG:
            syslog.syslog(self.syslog_prios[severity], msg)
        else:
            print "%s: %s" % (severity, msg)
        
    def debug(self, msg):
        if self.verbosity >= 3:
            self._log('DEBUG', msg)
        
    def info(self, msg):
        if self.verbosity >= 2:
            self._log('INFO', msg)
        
    def warn(self, msg):
        if self.verbosity >= 1:
            self._log('WARNING', msg)
        
    def error(self, msg):
        if self.verbosity >= 0:
            self._log('ERROR', msg)
            
    def fatal(self, msg):
        if self.verbosity >= 0:
            self._log('FATAL', msg)
            
            
class destination():
    COLLECTOR = 0
    QUEUE     = 1    
            

class CNLHTTPRequestHandler(BaseHTTPRequestHandler):        
        
    def add_package(self, package, urls):
        log.debug("Name: '%s'" % package)
        log.debug("Urls: '%s'" % urls)
                
        try:
            owner = client.user_id
            
            if package:
                client.addPackage(package, urls, owner, destination.dest)
            else:
                client.generateAndAddPackages(urls, owner, destination.dest)
        except:
            log.error("Failed to add package '%s'" % package)
        else:
            log.info("Successfully added package '%s'" % package)
    

    def get_post(self, name, default=""):
        if name in self.post:
            return self.post[name]
        else:
            return default

    def start_response(self, string):
        self.send_response(200)

        self.send_header("Content-Length", len(string))
        self.send_header("Content-Language", "de")
        self.send_header("Vary", "Accept-Language, Cookie")
        self.send_header("Cache-Control", "no-cache, must-revalidate")
        self.send_header("Content-type", "text/html")
        self.end_headers()

    def do_GET(self):
        path = self.path.strip("/").lower()
        #self.wfile.write(path+"\n")

        self.map = [ (r"add$", self.add),
                (r"addcrypted$", self.addcrypted),
                (r"addcrypted2$", self.addcrypted2),
                (r"flashgot", self.flashgot),
                (r"crossdomain\.xml", self.crossdomain),
                (r"checkSupportForUrl", self.checksupport),
                (r"jdcheck.js", self.jdcheck),
                (r"", self.flash) ]

        func = None
        for r, f in self.map:
            if re.match(r"(flash(got)?/?)?"+r, path):
                func = f
                break

        if func:
            try:
                resp = func()
                if not resp: resp = "success"
                resp += "\r\n"
                self.start_response(resp)
                self.wfile.write(resp)
            except Exception,e :
                self.send_error(500, str(e))
        else:
            self.send_error(404, "Not Found")

    def do_POST(self):
        form = FieldStorage(
                fp=self.rfile,
                headers=self.headers,
                environ={'REQUEST_METHOD':'POST',
                         'CONTENT_TYPE':self.headers['Content-Type'],
                         })

        self.post = {}
        for name in form.keys():
            self.post[name] = form[name].value

        return self.do_GET()

    def flash(self):
        return "JDownloader"

    def add(self):
        package = self.get_post('referer', 'ClickAndLoad Package')
        urls = filter(lambda x: x != "", self.get_post('urls').split("\n"))

        self.add_package(package, urls)

    def addcrypted(self):
        package = self.get_post('referer', 'ClickAndLoad Package')
        dlc = self.get_post('crypted').replace(" ", "+")

        self.client.upload_container(package, dlc)

    def addcrypted2(self):
        package = self.get_post("source", "ClickAndLoad Package")
        crypted = self.get_post("crypted")
        jk = self.get_post("jk")
        
        js = JsEngine()
        crypted = standard_b64decode(unquote(crypted.replace(" ", "+")))
        jk = "%s f()" % jk
        jk = js.eval(jk)
        Key = unhexlify(jk)
        IV = Key

        obj = AES.new(Key, AES.MODE_CBC, IV)
        result = obj.decrypt(crypted).replace("\x00", "").replace("\r","").split("\n")

        result = filter(lambda x: x != "", result)

        self.add_package(package, result)


    def flashgot(self):
        autostart = int(self.get_post('autostart', 0))
        package = self.get_post('package', "FlashGot")
        urls = filter(lambda x: x != "", self.get_post('urls').split("\n"))

        self.add_package(package, urls)

    def crossdomain(self):
        rep = "<?xml version=\"1.0\"?>\n"
        rep += "<!DOCTYPE cross-domain-policy SYSTEM \"http://www.macromedia.com/xml/dtds/cross-domain-policy.dtd\">\n"
        rep += "<cross-domain-policy>\n"
        rep += "<allow-access-from domain=\"*\" />\n"
        rep += "</cross-domain-policy>"
        return rep

    def checksupport(self):
        pass

    def jdcheck(self):
        rep = "jdownloader=true;\n"
        rep += "var version='10629';\n"
        return rep


class JsEngine():
    def __init__(self):
        self.engine = ENGINE
        self.init = False

    def __nonzero__(self):
        return False if not ENGINE else True

    def eval(self, script):
        if not self.init:
            if ENGINE == "pyv8":
                import PyV8
                global PyV8

            self.init = True

        if type(script) == unicode:
            script = script.encode("utf8")

        if not ENGINE:
            raise Exception("No JS Engine")
        
        if ENGINE == "pyv8":
            return self.eval_pyv8(script)
        elif ENGINE == "js":
            return self.eval_js(script)
        elif ENGINE == "rhino":
            return self.eval_rhino(script)
        
    def eval_pyv8(self, script):
        rt = PyV8.JSContext()
        rt.enter()
        return rt.eval(script)

    def eval_js(self, script):
        script = "print(eval(unescape('%s')))" % quote(script)
        p = subprocess.Popen(["js", "-e", script], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=-1)
        out, err = p.communicate()
        res = out.strip()
        return res

    def eval_rhino(self, script):
        script = "print(eval(unescape('%s')))" % quote(script)
        p = subprocess.Popen(["java", "-cp", rhinopath, "org.mozilla.javascript.tools.shell.Main", "-e", script],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=-1)
        out, err = p.communicate()
        res = out.strip()
        return res.decode("utf8").encode("ISO-8859-1")

        
################################################################################

def signal_handler(signum, frame):
    if signum == signal.SIGTERM:
        log.info("Received TERM signal, exiting")
        delete_pid_file()
        exit(0)
        
        
def write_pid_file():
    delete_pid_file()
    pid = os.getpid()
    f = open(__pid_file__, "wb")
    f.write(str(pid))
    f.close()


def delete_pid_file():
    if check_pid_file():
        log.debug("Deleting old .pid file %s" % __pid_file__)
        os.remove(__pid_file__)


def check_pid_file():    
    if os.path.isfile(__pid_file__):
        f = open(__pid_file__, "rb")
        pid = f.read().strip()
        f.close()
        if pid:
            pid = int(pid)
            return pid

    return 0


def is_already_running():
    pid = check_pid_file()
    if not pid or os.name == "nt": return False

    return pid


def quit_instance():
    if os.name == "nt":
        log.error("Not supported on windows.")
        return

    pid = is_already_running()
    if not pid:
        log.error("%s not running." % __app_name__)
        return

    try:
        os.kill(pid, signal.SIGTERM)

        t = time.time()
        log.info("Waiting for %s to quit." % __app_name__)

        while exists(__pid_file__) and t + 10 > time.time():
            time.sleep(0.25)

        if not exists(__pid_file__):
            log.info("%s successfully terminated." % __app_name__)
        else:
            os.kill(pid, signal.SIGKILL) #SIGKILL
            log.error("%s did not respond to TERM signal." % __app_name__)
            log.info("KILL signal was send to process with id %s." % pid)

    except Exception as e:
        log.error("Error quitting %s.\n%s" % (__app_name__, str(e)))
        
    
def daemonize():
    log.debug("Daemonizing")

    if (hasattr(os, "devnull")):
        REDIRECT_TO = os.devnull
    else:
        REDIRECT_TO = "/dev/null"

    try:     
        pid = os.fork()
    except OSError as e:
        raise Exception("%s [%d]" % (e.strerror, e.errno))
    
    if (pid == 0):    
        os.setsid()
      
        try:
            pid = os.fork()    # Fork a second child.
        except OSError as e:
            raise Exception("%s [%d]" % (e.strerror, e.errno))
        
        if (pid == 0):    # The second child.
            # Since the current working directory may be a mounted filesystem, we
            # avoid the issue of not being able to unmount the filesystem at
            # shutdown time by changing it to the root directory.
            os.chdir("/var/run")
            # We probably don't want the file mode creation mask inherited from
            # the parent, so we give the child complete control over permissions.
            os.umask(0)
        else:
            # exit() or _exit()?  See below.
            os._exit(0)    # Exit parent (the first child) of the second child.
    else:      
        os._exit(0)    # Exit parent of the first child.
            
    import resource        # Resource usage information.
    MAXFD = 1024
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if (maxfd == resource.RLIM_INFINITY):
        maxfd = MAXFD
    
    # Iterate through and close all file descriptors.
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError:    # ERROR, fd wasn't open to begin with (ignored)
            pass
        
    os.open(REDIRECT_TO, os.O_RDWR)    # standard input (0)
    
    # Duplicate standard input to standard output and standard error.
    os.dup2(0, 1)            # standard output (1)
    os.dup2(0, 2)            # standard error (2)
    
    signal.signal(signal.SIGTERM, signal_handler)
    
    write_pid_file()
    

def connect_to_core(host, port, username, password):
    log.info("Connecting to pyLoad at %s:%s as user '%s' with password." % (host, port, username))
    
    try:
        client = ThriftClient(host, port, username, password)
    except WrongLogin:
        raise Exception("Authentication with remote core failed. Invalid login credentials.")        
    except NoSSL:
        raise Exception("You need py-openssl to connect to this pyLoad Core.")
    except NoConnection:
        raise Exception("Could not connect to host %s. Is the pyLoadCore running?" % host)
    
    version = client.getServerVersion()
    log.info("Server version: %s" % version)
        
    client.user_id = client.getUserIDFromName(username)
    log.debug('Mapping user "%s" to user id "%s"' % (username, client.user_id))
    
    return client

    
def start_cnl_listener():
    log.info("Starting Click'n'Load server at localhost:9666.")
        
    server_address = ('localhost', 9666)        
    try:
        httpd = HTTPServer(server_address, CNLHTTPRequestHandler)
    except Exception as e:
        if e.errno == 98:
            log.fatal("Could not start Click'n'Load server. Address already in use.")
            exit(1)
    
    sa = httpd.socket.getsockname()
    log.debug(str(sa))
    log.info("%s ready to serve requests" % __app_name__)
    
    httpd.serve_forever()


def print_version():
    print "%s version %s - 2012 X3n0m0rph59" % (__app_name__, __version__)
    
    
def print_usage():
    print_version()
    print ""
    print "Usage: %s --host <pyLoadCore> --username <username> --password " \
          "<password>" % (sys.argv[0])
    print ""
    print "<Options>"
    print "  -c, --host\t\tConnect to host running pyLoadCore (required)"
    print "  -u, --username\tUsername for authentication with the remote core (required)"
    print "  -p, --password\tPassword for authentication with the remote core (required)"
    print "  -P, --port\t\tRemote port to connect to (default: 7227)"
    print "  -d, --destination\tDefault destination for added packages\n\t\t\t"\
                    "(either 'queue' or 'collector', default: collector)"
    print "  -D, --daemon\t\tDaemonize on startup  (default: no)"
    print "  -q, --quit\t\tQuit a running proxy instance"
    print "  -v, --verbosity\tSpecify output verbosity (Range: 0-3, default: %s)" % __log_verbosity__
    print "  -V, --version\t\tDisplay version information"
    print "  -h, --help\t\tDisplay this help"
    print "" 


def run():    
    try:
        try:
            opts, args = getopt.getopt(sys.argv[1:], "c:u:p:P:d:qDv:Vh", \
                                       ["host=", "username=", "password=", \
                                        "port=", "destination=", "daemon", \
                                        "quit", "verbosity=", "version", "help"])
            
            host = username = password = None
            port = 7227
            destination.dest = destination.COLLECTOR            
            do_daemonize = False
            
            for option, argument in opts:
                if option in ("-c", "--host"):
                    host = argument
                if option in ("-u", "--username"):
                    username = argument
                if option in ("-p", "--password"):
                    password = argument
                    
                if option in ("-P", "--port"):
                    port = int(argument)
                    
                if option in ("-d", "--destination"):
                    if argument.lower() == "queue":
                        destination.dest = destination.QUEUE
                    elif argument.lower() == "collector":    
                        destination.dest = destination.COLLECTOR
                    else:
                        raise Exception("Invalid destination specified, use either 'queue' or 'collector'.")
                    
                if option in ("-D", "--daemon"):
                    do_daemonize = True
                    log.target = log.SYSLOG
                if option in ("-q", "--quit"):
                    quit_instance()
                    exit(0)
                    
                if option in ("-v", "--verbosity"):
                    log.verbosity = int(argument)
                if option in ("-V", "--version"):
                    print_version()
                    exit(0)
                if option in ("-h", "--help"):
                    print_usage()
                    exit(0)
             
        except getopt.GetoptError as e:
            log.error(str(e))
            print_usage()        
            sys.exit(1)
        
        log.debug("%s Version %s starting up" % (__app_name__,__version__))
        
        if host is None or username is None or password is None:
            raise Exception("Please specify host, username and password!")
            
        if do_daemonize == True:
            daemonize()
           
        log.debug("Looking for a suitable JavaScript engine...")
        
        global ENGINE, JS, PYV8, RHINO
        ENGINE = ""
        JS = False
        PYV8 = False
        RHINO = False
    
        if not ENGINE:
            try:
                import subprocess
        
                subprocess.Popen(["js", "-v"], bufsize=-1, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
                p = subprocess.Popen(["js", "-e", "print(23+19)"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                out, err = p.communicate()
                #integrity check
                if out.strip() == "42":
                    ENGINE = "js"
                JS = True
            except:
                pass
        
        if not ENGINE:
            try:
                find_module("PyV8")
                ENGINE = "pyv8"
                PYV8 = True
            except:
                pass
        
        if not ENGINE:
            try:
                global rhinopath
                rhinopath = ""
                        
                if exists("/usr/share/java/js.jar"):
                    rhinopath = "/usr/share/java/js.jar"
                elif exists("js.jar"):
                    rhinopath = "js.jar"            
        
                if not rhinopath:
                    raise Exception
        
                import subprocess
        
                p = subprocess.Popen(["java", "-cp", rhinopath, "org.mozilla.javascript.tools.shell.Main", "-e", "print(23+19)"],
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                out, err = p.communicate()
                #integrity check
                if out.strip() == "42":
                    ENGINE = "rhino"
                RHINO = True
            except:
                pass
            
        if not ENGINE:
            log.error("No JavaScript engine found. Click'n'Load may not work correctly")
        else:
            log.info('Found JavaScript engine "%s"' % ENGINE)
            
        global client
        client = connect_to_core(host, port, username, password)
        
        start_cnl_listener()                    
        
    except KeyboardInterrupt:
        log.info("Received keyboard interrupt, exiting.")
        exit(1)
        
    except Exception as e:
        log.error(str(e))        
        log.fatal("Irrecoverable error occured, exiting now.")
        import traceback
        traceback.print_exc(25, None)        
        exit(1)


################################################################################

if __name__ == '__main__':    
    log = SimpleLogger()    
    run()